//********************************************************************
//*** ResourcePool/SQL/Factory.t.cc
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: Factory.t.cc,v 1.6 2007/10/13 08:57:11 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "t/main.hh"
#include "config.h"
#include "ResourcePool/SQL/Factory.hh"
#include "ResourcePool/ResourcePool.hh"
#include "ResourcePool/SQL/ResourcePool.hh"
#include "ResourcePool/SQL/SQLException.hh"

#include "testconfig.cc"

using namespace std;
using namespace fatalmind::SQL;

void TEST_SQL_Factory_ok() {
#ifdef HAVE_ORACLE
	SQLFactory<> f_oracle("oci:scott/tiger@test");
#endif
#ifdef HAVE_MYSQL
	SQLFactory<> f_mysql("mysql:test@test;example.com");
#endif
}

void TEST_SQL_Factory_wrong() {
	try {
		SQLFactory<> f("blahdoesn'texist:test@test;example.com");
		assertNotReach();
	} catch (fatalmind::CreateException e) {
	}
	try {
		SQLFactory<> f_oracle("oci9x");
		assertNotReach();
	} catch (fatalmind::CreateException e) {
	}
#ifdef HAVE_ORACLE
	try {
		SQLFactory<> f_oracle("oci9:");
		assertNotReach();
	} catch (fatalmind::CreateException e) {
	}
#endif
#ifdef HAVE_MYSQL
	try {
		SQLFactory<> f_oracle("mysql2:");
		assertNotReach();
	} catch (fatalmind::CreateException e) {
	}
#endif

}

void TEST_SQL_Factory_connect() {
#ifdef HAVE_ORACLE
	{
		string logon = "oci9:" + getOracleLogon();
		RP::Factory f(logon);	
		RP p(f);
		RP::PoolType::resource_t& r = p.get();
		p.free(r);
	}
#endif // HAVE_ORACLE
#ifdef HAVE_MYSQL
	{
		string logon = "mysql3:" + getMysqlLogon();
		RP::Factory f(logon);	
		RP p(f);
		RP::PoolType::resource_t& r = p.get();
		p.free(r);
	}
#endif // HAVE_MYSQL
}

