//********************************************************************
//*** ResourcePool/TaskRunner/TaskRunnerPool.hh
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: TaskRunnerPool.hh,v 1.15 2007/09/19 12:42:38 mws Exp $
//********************************************************************

// this class could grealy improve compile time by using Pimpl

#ifndef RESOURCEPOOL_TASKRUNNER_TASKRUNNERPOOL_HH
#define RESOURCEPOOL_TASKRUNNER_TASKRUNNERPOOL_HH

#include "mm/gc_ptr.hh"
#include "mm/CommonPtr.hh"

#ifndef RESOURCEPOOL_RESOURCEPOOL_HH
#include "ResourcePool/ResourcePool.hh"
#endif

#ifndef INCLUDED_TASKRUNNER_HH
#include "Thread/TaskRunner.hh"
#endif

#include "Thread/TaskRunnerInterface.hh"

#ifndef RESOURCEPOOL_TASKRUNNER_RESOURCE_HH
#include "Resource.hh"
#endif

#ifndef RESOURCEPOOL_TASKRUNNER_FACTORY_HH
#include "Factory.hh"
#endif

#ifndef INCLUDED_THREAD_LOCK_HH
#include "Thread/Lock.hh"
#endif

#include "Thread/LockStatistics.hh"

namespace fatalmind {

namespace TaskRunnerPoolInternals {
class CleanUpTask;
class ReleaseTaskRunner;
} // namespace TaskRunnerPoolInternals

/**
 * A special implementation for a pool of TaskRunner instances.
 */
class TaskRunnerPool: public TaskRunnerInterface {
	public:
		TaskRunnerPool(
			const ResourcePoolOptions& = ResourcePoolOptions::defaultOptions
		);
		~TaskRunnerPool();

		using TaskRunnerInterface::runDetached;
		virtual void runDetached(CommonPtr<Task, MultiThreadedModel<> >);

		using TaskRunnerInterface::run;
		TaskRunnerInterface::ID run(CommonPtr<Task, MultiThreadedModel<> >);

	private:
		typedef fatalmind::PoolType<TaskRunnerPoolInternals::Factory
			, TaskRunnerPoolInternals::Resource
			, fatalmind::TaskRunner
			, NopCommand<TaskRunnerPoolInternals::Factory>
			, MultiThreadedModel<UsageOnlyLockStatistics> > TaskRunnerPool_t;
		typedef ResourcePool<TaskRunnerPool_t> pool_t;

		TaskRunnerPool_t::factory_t _factory;
		pool_t _pool;

	// for cleanup
	private:
		void free(TaskRunnerPoolInternals::Resource&);
	friend
		class TaskRunnerPoolInternals::CleanUpTask;	
	friend
		class TaskRunnerPoolInternals::ReleaseTaskRunner;	

	// hidden stuff	
	private:
		TaskRunnerPool(const TaskRunnerPool&);
		TaskRunnerPool& operator=(const TaskRunnerPool&);
};

} // namespace fatalmind
#endif

