//********************************************************************
//*** ResourcePool/mysql/Factory.hh
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: Factory.hh,v 1.10 2007/10/13 08:57:11 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */


#ifndef RESOURCEPOOL_MYSQL_FACTORY_HH
#define RESOURCEPOOL_MYSQL_FACTORY_HH

#ifndef RESOURCEPOOL_MYSQL_RESOURCE_HH
#include "Resource.hh"
#endif

#include "ResourcePool/SQL/SQLFactoryInterface.hh"

#include <string>

namespace fatalmind {

class mysqlFactory : public SQL::SQLFactoryInterface {
	typedef SQL::SQLFactoryInterface super;
	public:
		mysqlFactory(const std::string& host
			, const std::string& user
			, const std::string& pass
			, const std::string& db
			, unsigned int port = 3306
			, const std::string& sock = ""
		);
		mysqlFactory(const std::string& connect);

		using super::databaseType_t;
		databaseType_t getDatabaseType() const {
#ifdef HAVE_MYSQL_TRANSACTIONS
			return MYSQLTX;
#else
			return MYSQL;
#endif
		};
	
		mysqlResource* create_resource() const;
			
	protected:
		virtual Clone* DoClone() const;

	private:
		std::string _host;
		std::string _user;
		std::string _pass;
		std::string _db;
		unsigned int _port;
		const std::string _sock;
};

} // namespace fatalmind
#endif
