//********************************************************************
//*** Thread/ThreadRunnerInterfacehh
//*** Copyright (c) 2003-2007 by Markus Winand <mws@fatalmind.com>
//*** $Id: TaskRunnerInterface.hh,v 1.5 2007/09/06 18:16:49 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef THREAD_TASKRUNNERINTERFACE_HH
#define THREAD_TASKRUNNERINTERFACE_HH

#include "Thread/Task.hh"

#include "mm/gc_ptr.hh"
#include "mm/CommonPtr.hh"

namespace fatalmind {

class WaitObject;

class TaskRunnerInterface {
	public:
		class ID {
			public:
				ID();
				ID(const ID&);
				ID(gc_ptr<WaitObject, MultiThreadedModel<> >&);
				~ID() throw();

				ID& operator=(const ID&);
				void wait();
				void detach();
			private:
				gc_ptr<WaitObject, MultiThreadedModel<> > _wo;
		};


		template<class D>
		inline void runDetached(std::auto_ptr<D> a) {
			runDetached(std::auto_ptr<Task>(a));
		}
		template<class D>
		inline void runDetached(gc_ptr<D, MultiThreadedModel<> > a) {
			runDetached(gc_ptr<Task, MultiThreadedModel<> >(a));
		}
		inline void runDetached(std::auto_ptr<Task> a) {
			CommonPtr<Task, MultiThreadedModel<> > c(a);
			runDetached(c);
		}
		inline void runDetached(gc_ptr<Task, MultiThreadedModel<> > a) {
			CommonPtr<Task, MultiThreadedModel<> > c(a);
			runDetached(c);
		}


		template<class D>
		inline ID run(std::auto_ptr<D> a) {
			return run(std::auto_ptr<Task>(a));
		}
		template<class D>
		inline ID run(gc_ptr<D, MultiThreadedModel<> > a) {
			return run(gc_ptr<Task, MultiThreadedModel<> >(a));
		}
		inline ID run(std::auto_ptr<Task> a) {
			CommonPtr<Task, MultiThreadedModel<> > c(a);
			return run(c);
		}
		inline ID run(gc_ptr<Task, MultiThreadedModel<> > a) {
			CommonPtr<Task, MultiThreadedModel<> > c(a);
			return run(c);
		}

	protected:
        virtual void runDetached(CommonPtr<Task, MultiThreadedModel<> >) = 0;
        virtual ID run(CommonPtr<Task, MultiThreadedModel<> >) = 0;

		virtual ~TaskRunnerInterface() {
		}
};

} // namespace fatalmind
#endif
